"""RIOT pkg generator module."""

import os
import os.path
import datetime

import click
from click import MissingParameter

from .helpers import _get_usermail, _get_username
from .helpers import TEMPLATES_DIR
from .helpers import _read_config, _parse_list_option
from .helpers import _prompt_common_information, _check_common_params
from .helpers import generate_source, generate_file


def _read_pkg_config(filename):
    """Read the pkg specific configuration file."""
    params = _read_config(filename, section='pkg')
    if 'name' not in params or not params['name']:
        raise MissingParameter(param_type='package name')
    if 'displayed_name' not in params or not params['displayed_name']:
        raise MissingParameter(param_type='board displayed name')
    if 'url' not in params or not params['url']:
        raise MissingParameter(param_type='package source url')
    if 'hash' not in params or not params['hash']:
        raise MissingParameter(param_type='package version hash')
    if 'license' not in params or not params['license']:
        raise MissingParameter(param_type='package license')
    if 'description' not in params:
        params['description'] = ''
    return params


def _prompt_pkg_params():
    """Request pkg specific variables."""
    params = {}
    params['name'] = click.prompt(text='Package name (no space)')
    params['displayed_name'] = click.prompt(
        text='Package displayed name (for doxygen documentation)')
    params['url'] = click.prompt(text='Package source url')
    params['hash'] = click.prompt(text='Package version hash')
    params['license'] = click.prompt(text='Package license')
    params['description'] = click.prompt(text='Package short description')

    params.update(_prompt_common_information())
    return params


def _check_pkg_params(params):
    test_name = params['name'].replace(' ', '_')
    params['name'] = test_name


def generate_pkg(config=None):
    # Start wizard if config is not set
    if config is None:
        params = _prompt_pkg_params()
    else:
        params = _read_pkg_config(config)
    _check_pkg_params(params)
    _check_common_params(params)

    pkgs_dir = os.path.join(os.path.expanduser(params['riotbase']), 'pkg')
    pkg_dir = os.path.join(pkgs_dir, params['name'])

    riotbase = os.path.abspath(os.path.expanduser(params['riotbase']))
    if os.path.abspath(os.path.curdir) == riotbase:
        params['output_dir'] = os.path.join('pkg', params['name'])
    else:
        params['output_dir'] = os.path.expanduser(pkg_dir)

    if not os.path.exists(pkg_dir):
        os.makedirs(pkg_dir)
    elif not click.prompt('\'{name}\' pkg directory already exists, '
                          'overwrite (y/N)?'.format(**params),
                          default=False, show_default=False):
        click.echo('Abort')
        return

    generate_source(params, 'pkg', ['doc.txt',
                                    'Makefile',
                                    'Makefile.dep',
                                    'Makefile.include'])

    makefile_pkg_in = os.path.join(TEMPLATES_DIR, 'pkg', 'Makefile.pkg')
    makefile_pkg_out = os.path.join(params['output_dir'],
                                    'Makefile.{}'.format(params['name']))
    generate_file(params, makefile_pkg_in, makefile_pkg_out)

    click.echo(click.style('Package \'{name}\' generated in '
                           '{output_dir} with success!'
                           .format(**params), bold=True))



import logging
import click

from riotgen.application import generate_application
from riotgen.example import generate_example
from riotgen.board import generate_board
from riotgen.pkg import generate_pkg
from riotgen.test import generate_test

logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s - %(name)14s - '
                           '%(levelname)5s - %(message)s')
logger = logging.getLogger("riotgen.generator")


@click.group()
def cli():
    pass


@cli.command(help='Bootstrap a RIOT application')
@click.argument('output_dir', type=click.Path(exists=True))
@click.option('--config', type=click.File(mode='r'),
              help='Application initial configuration file')
def application(output_dir, config):
    generate_application(output_dir, config)


@cli.command(help='Bootstrap a RIOT board support')
@click.option('--config', type=click.File(mode='r'),
              help='Board support initial configuration file')
def board(config):
    generate_board(config)


@cli.command(help='Bootstrap a RIOT example application')
@click.option('--config', type=click.File(mode='r'),
              help='Example application initial configuration file')
def example(config):
    generate_example(config)


@cli.command(help='Bootstrap a RIOT external package')
@click.option('--config', type=click.File(mode='r'),
              help='Package initial configuration file')
def pkg(config):
    generate_pkg(config)


@cli.command(help='Bootstrap a RIOT test application')
@click.option('--config', type=click.File(mode='r'),
              help='Test application initial configuration file')
def test(config):
    generate_test(config)


if __name__ == '__main__':
    cli()


"""RIOT example application generator module."""

import os.path
import datetime

import click
from click import MissingParameter

from .helpers import _get_usermail, _get_username
from .helpers import TEMPLATES_DIR
from .helpers import _read_config, _parse_list_option
from .helpers import _prompt_common_information, _check_common_params
from .helpers import generate_application_source


def _read_example_config(filename):
    """Read the application specific configuration file."""
    params = _read_config(filename, section='example')
    if 'name' not in params or not params['name']:
        raise MissingParameter(param_type='example application name')
    if 'brief' not in params:
        params['brief'] = ''
    if 'board' not in params:
        params['board'] = 'native'
    else:
        params['board'] = params['board']
    for param in ['modules', 'packages', 'features']:
        if param not in params:
            params[param] = ''
        else:
            params[param] = _parse_list_option(params[param])
    return params


def _prompt_example_params():
    """Request application specific variables."""
    params = {}
    params['name'] = click.prompt(
        text='Example application name (no space)')
    params['brief'] = click.prompt(
        text='Example application brief description', default='')
    params['board'] = click.prompt(text='Target board', default='native')
    params['modules'] = click.prompt(
        text='Required modules (comma separated)', default='',
        value_proc=_parse_list_option)
    params['packages'] = click.prompt(
        text='Required packages (comma separated)', default='',
        value_proc=_parse_list_option)
    params['features'] = click.prompt(
        text='Required board features (comma separated)', default='',
        value_proc=_parse_list_option)

    params.update(_prompt_common_information())
    return params


def _check_example_params(params):
    application_name = params['name'].replace(' ', '_')
    params['name'] = application_name
    params['name_underline'] = '=' * len(application_name)
    params['includes'] = ''
    for module in params['modules']:
        params['includes'] += 'USEMODULE += {}\n'.format(module)
    for package in params['packages']:
        params['includes'] += 'USEPKG += {}\n'.format(package)
    for feature in params['features']:
        params['includes'] += 'FEATURES_REQUIRED += {}\n'.format(feature)


def generate_example(config=None):
    # Start wizard if config is not set
    if config is None:
        params = _prompt_example_params()
    else:
        params = _read_example_config(config)
    _check_example_params(params)
    _check_common_params(params)

    examples_dir = os.path.join(os.path.expanduser(params['riotbase']),
                                                   'examples')
    example_dir = os.path.join(examples_dir, params['name'])
    riotbase = os.path.abspath(os.path.expanduser(params['riotbase']))
    if os.path.abspath(os.path.curdir) == riotbase:
        params['output_dir'] = os.path.join('examples', params['name'])
    else:
        params['output_dir'] = os.path.expanduser(example_dir)

    if not os.path.exists(example_dir):
        os.makedirs(example_dir)
    elif not click.prompt('\'{name}\' example directory already exists, '
                          'overwrite (y/N)?'.format(**params),
                          default=False, show_default=False):
        click.echo('Abort')
        return

    generate_application_source(params, template_dir='example')

    click.echo(click.style('Application \'{name}\' generated in {output_dir} '
                           'with success!'.format(**params), bold=True))
